﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace SmartBackgroundWorker
{
  /// <summary>
  /// Simple demonstration of the SmartBackgroundWorker class.
  /// </summary>
  public partial class MainPage : UserControl
  {
    public MainPage()
    {
      InitializeComponent();
    }

    /// <summary>
    /// Local instance of SmartBackgroundWorker.
    /// </summary>
    SmartBackgroundWorker _smartBackgroundWorker = new SmartBackgroundWorker();

    /// <summary>
    /// Flag to stop operation when it was canceled by user.
    /// </summary>
    bool _stop;

    /// <summary>
    /// A simple function that gets alternating executed by
    /// the UI-thread and a background-thread.
    /// </summary>
    IEnumerator<SwitchTo> MyWorkerFunktion(string someParameter, int someOtherParameter)
    {
      // At start we run in the background thread, so switch to UI-thread
      // to update the StartStopButton and clear the ListBox
      yield return SwitchTo.UIThread;
      StartStopButton.Content = "Stop";
      Listbox.Items.Clear();

      // Now switch to background thread and start the loop
      yield return SwitchTo.BackgroundThread;
      for (int idx = 1; idx <= 25; idx++)
      {
        // Get the next item
        string item = DoSomeHardWorkToGetAnItem(idx);

        // Switch to UI-thread to update the list-box
        yield return SwitchTo.UIThread;
        Listbox.Items.Add(item);

        // Check if user pressed the Stop button
        if (_stop)
        {
          Listbox.Items.Add("<Operation was canceled by user>");
          _stop = false;
          break;
        }

        // Switch back to background thread for next loop iteration
        yield return SwitchTo.BackgroundThread;
      }

      // We have done and reset Start/Stop button in UI-thread
      yield return SwitchTo.UIThread;
      StartStopButton.Content = "Start";
    }

    /// <summary>
    /// Simulates a time consuming background operation.
    /// </summary>
    static string DoSomeHardWorkToGetAnItem(int number)
    {
      const int delay = 750;
      DateTime now = DateTime.Now;
      while ((DateTime.Now - now).TotalMilliseconds < delay)
      { /* Active wait */ }

      return String.Format("Item number #{0:00}", number);
    }

    /// <summary>
    /// Handle Start/Stop button.
    /// </summary>
    private void StartStop_Click(object sender, RoutedEventArgs e)
    {
      if (!_smartBackgroundWorker.IsBusy)
      {
        // Call worker function with some arbitrary parameters
        _smartBackgroundWorker.RunWorkerAsync(MyWorkerFunktion("Hello", 42));
      }
      else
      {
        _stop = true;
      }
    }
  }
}
